///////////////////////////////////////////////////////////////////////////
//
//	SDL_CheckSB() - Checks to see if a SoundBlaster resides at a
//		particular I/O location
//
///////////////////////////////////////////////////////////////////////////

//For outport/inport functions
#include<inlines/pc.h> 

//Kolibri & Menuet system servis functions
#include<menuet/os.h>
#include "kolibri.h"


#define boolean unsigned char
#define true 1
#define false 0


// 	Registers for the Sound Blaster card - needs to be offset by n0 (0x10,0x20,0x30,0x40,0x50,0x60)
#define	sbReset		0x206	// W
#define	sbFMStatus	0x208	// R
#define	sbFMAddr	0x208	// W
#define	sbFMData	0x209	// W
#define	sbReadData	0x20a	// R
#define	sbWriteCmd	0x20c	// W
#define	sbWriteData	0x20c	// W
#define	sbWriteStat	0x20c	// R
#define	sbDataAvail	0x20e	// R

static int	sbLocation = -1;

// Macros for SoundBlaster stuff
#define	sbOut(n,b)	outportb((n) + sbLocation,b)
#define	sbIn(n)		inportb((n) + sbLocation)
#define	sbWriteDelay()	while (sbIn(sbWriteStat) & 0x80);
#define	sbReadDelay()	while (sbIn(sbDataAvail) & 0x80);


//_________________________________________________________________________________
#define RESETS 10
#define RESETTIMES 50

/* I/O Port offsets. */
#define DSPResetPortOffset 0x06  // DSP Reset port.                  Write Only.
#define DSPReadPortOffset  0x0A  // DSP Read data port.              Read  Only.
#define DSPLifePortOffset  0x0A  // DSP Read data port.              Read  Only.
#define DSPWStatPortOffset 0x0C  // DSP Write buffer status port.    Write Only.
#define DSPWritePortOffset 0x0C  // DSP Write data port.             Write Only.
#define DSPRStatPortOffset 0x0E  // DSP Read buffer status port.     Read  Only.
#define DSP8AckPortOffset  0x0E  //  8 bit DMA IRQ Acknowledge port. Write Only.
#define DSP16AckPortOffset 0x0F  // 16 bit DMA IRQ Acknowledge port. Write Only.

/************************************************************************}
{ Reset the soundblaster (if present) and get the correct I/O-port       }
{ This routine works with all soundblaster card versions                */

int wsbdetect(void)
{
  unsigned nr_resets, times_reset;
  int sbdetected = 0;
	int io_port; //ASPER

  io_port   = 0x210;
  nr_resets = RESETS;
  while ( (io_port <= 0x260) && (!sbdetected) )
  {
    outportb(io_port + DSPResetPortOffset, 1);
    outportb(io_port + DSPResetPortOffset, 0);
    times_reset = RESETTIMES;
    while ((times_reset > 0) && (inportb(io_port + DSPRStatPortOffset) < 128)) times_reset--;
    if ( (times_reset == 0) || (inportb(io_port + DSPReadPortOffset) != 0xAA) )
    {
      nr_resets--;
      if (nr_resets==0)
      {
	nr_resets = RESETS;
	io_port += 0x10;
      }
    }
    else
      sbdetected = 1;
  }
//  if (sbdetected) getdspversion();
  return sbdetected;
}
//_________________________________________________________________________________

static boolean
SDL_CheckSB(int port)
{
	int	i;

	sbLocation = port << 4;		// Initialize stuff for later use

	sbOut(sbReset,true);		// Reset the SoundBlaster DSP
	__asm__ __volatile__(
		"movw $0x388, %%dx\n\t"		// Wait >4usec
		"movl $9, %%ecx\n\t"
		"1:\tinb %%dx, %%al\n\t"
		"loop 1b"
		::
	);
	sbOut(sbReset,false);		// Turn off sb DSP reset
	__asm__ __volatile__(
		"movw $0x388, %%dx\n\t"		// Wait >100usec
		"movl $100, %%ecx\n\t"
		"1:\tinb %%dx, %%al\n\t"
		"loop 1b"
		::
	);


	for (i = 0;i < 100;i++)
	{
		if (sbIn(sbDataAvail) & 0x80)		// If data is available...
		{
			if (sbIn(sbReadData) == 0xaa)	// If it matches correct value
				return(true);
			else
			{
				sbLocation = -1;			// Otherwise not a SoundBlaster
				return(false);
			}
			__kolibri__debug_out("Data available.\n\r\0");
		}
//		__kolibri__debug_out("I");
	}
	sbLocation = -1;						// Retry count exceeded - fail
	return(false);
}


///////////////////////////////////////////////////////////////////////////
//
//	Checks to see if a SoundBlaster is in the system. If the port passed is
//		-1, then it scans through all possible I/O locations. If the port
//		passed is 0, then it uses the default (2). If the port is >0, then
//		it just passes it directly to SDL_CheckSB()
//
///////////////////////////////////////////////////////////////////////////
static boolean
SDL_DetectSoundBlaster(int port)
{
	int	i;

	if (port == 0)					// If user specifies default, use 2
		port = 2;
	if (port == -1)
	{
		if (SDL_CheckSB(2))			// Check default before scanning
			return(true);

		if (SDL_CheckSB(4))			// Check other SB Pro location before scan
			return(true);

		for (i = 1;i <= 6;i++)		// Scan through possible SB locations
		{
			if ((i == 2) || (i == 4))
				continue;

			if (SDL_CheckSB(i))		// If found at this address,
				return(true);		//	return success
		}
		return(false);				// All addresses failed, return failure
	}
	else
		return(SDL_CheckSB(port));	// User specified address or default
}

void app_main(void)
{
	int ports_reserve_error=1;
	int sbBasePort=-1, sbFirstBasePort=0x206, sbLastBasePort=0x20E;
	
	sbLastBasePort+=6<<4;//sbBasePort<<4;
	//paint();
	ports_reserve_error = __kolibri__reserve_ports(0x388,0x389,0);
	if (ports_reserve_error) 
	{
		__kolibri__debug_out("\n\rCan't reserve port 0x388.\n\r\0");
		//return;
	}
	ports_reserve_error = __kolibri__reserve_ports(sbFirstBasePort,sbLastBasePort,0);
	if (ports_reserve_error) 
	{
		__kolibri__debug_out("\n\rCan't reserve 0x206-0x2XE ports.\n\r\0");
		//return;
	}	

	if (SDL_DetectSoundBlaster(2)==true) 
		__kolibri__debug_out("\n\r1. Sound Blaster found.\n\r\0");
	else
		__kolibri__debug_out("\n\r1. Sound Blaster not found.\n\r\0");
		
	if (wsbdetect()==true) 
		__kolibri__debug_out("\n\r2. Sound Blaster found.\n\r\0");
	else
		__kolibri__debug_out("\n\r2. Sound Blaster not found.\n\r\0");
		
	__kolibri__reserve_ports(sbFirstBasePort,sbLastBasePort,1);
	__kolibri__reserve_ports(0x388,0x389,1);
}
